#ifdef SONY_DSCF1

#include "config.h"
#include <stdio.h>
#include <sys/types.h>
#include "tty.h"
#include <termios.h>
#include <unistd.h>

void	Exit(int);

#define	BOFRAME 0xC0
#define	EOFRAME 0xC1
#define CESCAPE 0x7D

extern int Camfd;
static u_char address = 0;
static u_char sendaddr[8] = { 0x00, 0x22, 0x44, 0x66, 0x88, 0xaa, 0xcc, 0xee };
static u_char recvaddr[8] = { 0x0e, 0x20, 0x42, 0x64, 0x86, 0xa8, 0xca, 0xec };
static int sw_mode = 0;
static int pic_num = 0;
static int pic_num2 = 0;
static int year, month, date;
static int hour, minutes;

u_char
checksum(u_char addr, u_char *cp, int len)
{
    int ret = addr;
    while(len --)
	ret = ret + (*cp++);
    return  0x100-(ret & 0xff);
}

void 
sendcommand(u_char *p, int len)
{
    wbyte(BOFRAME);
    wbyte(sendaddr[address]);
    wstr(p, len);
    wbyte(checksum(sendaddr[address], p, len));
    wbyte(EOFRAME);
    address++; 
    if(address > 7) address = 0;
}

void
Abort()
{
    u_char buf[4];
    buf[0] = BOFRAME;
    buf[1] = 0x85;
    buf[2] = 0x7B;
    buf[3] = EOFRAME;
    wstr(buf, 4);
}  

int
recvdata(u_char *p, int len)
{
    u_char s, t;
    int sum;
    int i;

    s = rbyte();  /* BOFL */
    t= rbyte();  /* recvaddr */

    if(t != recvaddr[address]){
	s = rbyte();  /* drain */
	s = rbyte();  /* drain */
	s = rbyte();  /* drain */
	Abort();
	return -1;
    }
    i = len;
    sum = (int) t;
    while ((s = rbyte()) != EOFRAME){
	sum = sum + s;
	if(i > 0){
	    if(s == CESCAPE){
		s = rbyte();
		if(0x20 & s)
		    s = 0xDF & s;
		else
		    s = 0x20 | s;
	    }
	    *p = s;
	    p++;
	    i--;
	}
	t = s;
    }
    if(sum & 0xff){
#ifdef DEBUG
	fprintf(stderr,"check sum error.(%02x)\n", sum);
#endif
	return -1;
    }
    return len-i;
}

/*------------------------------------------------------------*/

int
CamReset()
{
    u_char buf[3];
retryreset:
    buf[0] = 0x01;
    buf[1] = 0x02;
    sendcommand(buf, 2);
    recvdata(buf, 3);
    if(!((buf[0] == 0x01 ) && (buf[1] == 0x02) && buf[2] == 0x00))
	goto retryreset;
    return (int) buf[2];		/*ok*/
}

int
CamOk()
{
    u_char buf[64];
    int i = 2;

    while (i--) {
	buf[0] = 0x01;
	buf[1] = 0x01;
	sprintf(&buf[2],"SONY     MKY-1001         1.00");
	sendcommand(buf, 32);
	recvdata(buf, 32);
	if((buf[0] != 0x01) || (buf[1] != 0x01) || (buf[2] != 0x00) ) {
	    Abort();
	    CamReset();
	} else
	    return 0;
    }
    return -1;
}

int
CamStatus(int verbose)
{
    u_char buf[34];
    int i;

    buf[0] = 0x03;
    buf[1] = 0x02;
    sendcommand(buf, 2);
    i = recvdata(buf, 33);
#ifdef DEBUG
    fprintf(stderr,"Status: %02x%02x:%02x(len = %d)\n", buf[0], buf[1], buf[2], i);
#endif
    if((buf[0] != 0x03) || (buf[1] != 0x02) ||(buf[2] != 0)){
	Abort();
	return -1;
    }

    sw_mode = buf[3];
    pic_num = buf[4] * 0x100 + buf[5];
    pic_num2 = buf[6] * 0x100 + buf[7];
    year = (buf[10] >> 4 ) * 10 + (buf[10] & 0x0f);
    month = (buf[11] >> 4 ) * 10 + (buf[11] & 0x0f);
    date = (buf[12] >> 4 ) * 10 + (buf[12] & 0x0f);
    hour = (buf[13] >> 4 ) * 10 + (buf[13] & 0x0f);
    minutes = (buf[14] >> 4 ) * 10 + (buf[14] & 0x0f);

    return buf[2];		/*ok*/
}

int
CamHowmany()
{
    CamStatus(0);
    return pic_num;
}

int
CamFopen(char *name)
{
    u_char buf[64];
    int len;

    buf[0] = 0x02;
    buf[1] = 0x0A;
    buf[2] = 0x00;
    buf[3] = 0x00;
    sprintf(&buf[4], "%s", name);
    len = strlen(name) + 5;
    sendcommand(buf, len);
    recvdata(buf, 6);
    if((buf[0] != 0x02) || (buf[1] != 0x0A) || (buf[2] != 0x00)){
	Abort();
	fprintf(stderr,"F1fopen fail\n");
	return -1;
    }

    return buf[3];
}

int
CamFclose()
{
    u_char buf[4];

    buf[0] = 0x02;
    buf[1] = 0x0B;
    buf[2] = 0x00;
    buf[3] = 0x00;
    sendcommand(buf, 4);
    recvdata(buf, 3);
    /*  fprintf(stderr,"Fclose: %02x%02x:%02x\n", buf[0], buf[1], buf[2]); */
    if((buf[0] != 0x02) || (buf[1] != 0x0B) || (buf[2] != 0x00)){
	fprintf(stderr,"F1fclose fail\n");
	Abort();
	return -1;
    }
    return buf[2];		/* ok == 0 */
}

long
CamFread(u_char *data, long len)
{

    long len2;
    long i = 0;
    u_char s;

    u_char buf[10];

    buf[0] = 0x02;
    buf[1] = 0x0C;
    buf[2] = 0x00;
    buf[3] = 0x00;
    buf[4] = 0;
    buf[5] = 0;

    buf[6] = (len >> 8) & 0xff;
    buf[7] = 0xff & len;

    sendcommand(buf, 8);
    rstr(buf, 9);
    if((buf[2] != 0x02) || (buf[3] != 0x0C) || (buf[4] != 0x00)){
	Abort();
	fprintf(stderr,"F1fread fail\n");
	return -1;
    }

    len2 = buf[7] * 0x100 + buf[8]; /* data size */
    if(len2 == 0) {
	s = rbyte(); /* last block checksum */
	s = rbyte(); /* last block EOFL */
	return 0;
    }  
    while((s = rbyte()) != EOFRAME){
	if(s == CESCAPE){
	    s = rbyte();
	    if(0x20 & s)
		s = 0xDF & s;
	    else
		s = 0x20 | s;
	}
	if(i < len)
	    data[i] = s;
	i++;

    }
    i--; /* checksum */
    return i; 
}

long
CamFseek(long offset, int base)
{
    u_char buf[10];

    buf[0] = 0x02;
    buf[1] = 0x0E;
    buf[2] = 0x00;
    buf[3] = 0x00;

    buf[4] = (offset >> 24) & 0xff;
    buf[5] = (offset >> 16) & 0xff;
    buf[6] = (offset >> 8) & 0xff;
    buf[7] = 0xff & offset;

    buf[8] = (base >> 8) & 0xff;
    buf[9] = 0xff & base;

    sendcommand(buf, 10);
    recvdata(buf, 3);
    if((buf[0] != 0x02) || (buf[1] != 0x0E) || (buf[2] != 0x00)){
	Abort();
	return -1;
    }

    return buf[2];
}

u_long
CamFinfo(char *name)
{
    u_char buf[64];
    int len;
    u_long flen;

    buf[0] = 0x02;
    buf[1] = 0x0F;
    sprintf(&buf[2], "%s", name);
    len = strlen(name) + 3;

    sendcommand(buf, len);
    len = recvdata(buf, 37);
    if((buf[0] != 0x02) || (buf[1] != 0x0F) || (buf[2] != 00)){
	Abort();
	return 0;
    }

    flen = buf[33] * 0x1000000 + buf[34] * 0x10000 + buf[35] * 0x100 + buf[36];

    if(buf[2] != 0) return(0);

    return flen;
}


long
CamGetdata(char *name, u_char *data, int verbose)
{
    long filelen;
    long total = 0;
    long len;
    u_char *p;

    CamStatus(0);
    p = data;
    filelen = CamFinfo(name);
    if(filelen < 0) 
	return 0;

    if(CamFopen(name) != 0)
	return 0;

    while((len = CamFread(p, 0x0400)) != 0){
	if(len < 0){
	    CamFclose();
	    return 0;
	}
	p = p + len;
	total = total + len;
	if(verbose)
	    fprintf(stderr, "%6u/%6u\b\b\b\b\b\b\b\b\b\b\b\b\b", (unsigned)total, (unsigned)filelen);
    }
    CamFclose();
    if(verbose)
	fprintf(stderr, "\n");
    return total;
}

int
CamDeletepicture(int n)
{
    u_char buf[4];
    buf[0] = 0x02;
    buf[1] = 0x15;
    buf[2] = 0x00;
    buf[3] = 0xff & n;
    sendcommand(buf, 4);
    recvdata(buf, 3);
    if((buf[0] != 0x02) || (buf[1] != 0x15) || (buf[2] != 0)){
	Abort();
	return -1;
    }
    return 0;
}


int
F1test(int n)
{
    u_char buf[4];
    buf[0] = 0x03;
    buf[1] = 0x15;
    buf[2] = 0x00;
    buf[3] = 0xff & n;
    /*  sendcommand(buf, 4); */
    sendcommand(buf, 2);
    recvdata(buf, 3);
    if((buf[0] != 0x03) || (buf[1] != 0x15) || (buf[2] != 0)){
	Abort();
	fprintf(stderr,"abort(%02x %02x %02x)\n", buf[0], buf[1], buf[2]);
	return -1;
    }
    return 0;
}

#endif
