#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <X11/xpm.h>
#include "main.h"

void
DrawThumb(int pos, int show)
{
    int cols, rows;
    int x,y;

    cols = curw/(THUMB_W+50);
    rows = (curh-190)/(THUMB_H+50);

    if(pos > (cols*rows)-1)
	return;

    x=(THUMB_W+50)*(pos%cols)+20;
    y=(THUMB_H+50)*(pos/cols)+130+20;

    XPutImage(display,background,gc,thumbnails[pos].thumbxim, 0,0, x, y, THUMB_W,THUMB_H);
    if(show)
	XPutImage(display,window,gc,thumbnails[pos].thumbxim, 0,0, x, y, THUMB_W,THUMB_H);
}

void
DrawFrame(int pos, int hilight, int show)
{
    int cols, rows;
    int x,y;

    cols = curw/(THUMB_W+50);
    rows = (curh-190)/(THUMB_H+50);

    if (pos > (cols*rows)-1)
	return;

    x=(THUMB_W+50)*(pos%cols);
    y=(THUMB_H+50)*(pos/cols)+130;

    if (hilight) {
	XCopyArea(display,hiframe,background,gc,0,0,THUMB_W+20,THUMB_H+20,x+10,y+10);
	if(show)
	    XCopyArea(display,hiframe,window,gc,0,0,THUMB_W+20,THUMB_H+20,x+10,y+10);
    } else {
	XCopyArea(display,frame,background,gc,0,0,THUMB_W+20,THUMB_H+20,x+10,y+10);
	if(show)
	    XCopyArea(display,frame,window,gc,0,0,THUMB_W+20,THUMB_H+20,x+10,y+10);
    }
}

void
Tilebg(int w, int h)
{
    int i,j,x,y;

    y=0;
    while (h) {
	if (h < 256)
	    j = h;
	else
	    j = 256;
	for (x=0,i=w; i;) {
	    if (i < 256) {
		XCopyArea(display,bgtile,background,gc,0,0,i,j,x,y);
		x += i;
		i = 0;
	    } else {
		XCopyArea(display,bgtile,background,gc,0,0,256,j,x,y);
		i -= 256;
		x += 256;
	    }
	}
	if (h < 256) {
	    y += h;
	    h = 0;
	} else {
	    h -= 256;
	    y += 256;
	}
    }
/*    
    194,94
    166,99
    131,123
 */    

    XSetClipMask(display, gc, retxpmmask);
    i = curw/6-87;
    XSetClipOrigin(display, gc, i, 12);
    XCopyArea(display,retxpm,background,gc,0,0,194,94,i,12);

    XSetClipMask(display, gc, deletexpmmask);
    i = curw*3/6-83+20;
    XSetClipOrigin(display, gc, i, 12);
    XCopyArea(display,deletexpm,background,gc,0,0,166,99,i,12);

    XSetClipMask(display, gc, quitxpmmask);
    i = curw*5/6-65;
    XSetClipOrigin(display, gc, i, 3);
    XCopyArea(display,quitxpm,background,gc,0,0,131,123,i,3);

    XSetClipMask(display, gc, None);
    XCopyArea(display,slidexpm,background,gc,0,0,272,42,curw/2-136,curh-51);
}

void
XPMerr(int xpm_err)
{
    switch (xpm_err) {
	case XpmOpenFailed:
	    fprintf(stderr,"Xpm file open failed!\n");
	    exit(1);
	case XpmFileInvalid:
	    fprintf(stderr,"Invalid Xpm file!\n");
	    exit(1);
	case XpmNoMemory:
	    fprintf(stderr,"Not enough memory !\n");
	    exit(1);
	case XpmColorFailed:
	    fprintf(stderr,"Can`t get any colors...\n");
	    exit(1);
	default:
	    break;
    }
}         

void
HandleExposeEvent(XExposeEvent *event)
{
    int x = event->x;
    int y = event->y;
    int w = event->width;
    int h = event->height;

    XCopyArea(display,background,window,gc,x,y,w,h,x,y);
}         

void InitWindow(int argc, char *argv[])
{
    XSizeHints		size_hints;
    XWMHints		wm_hints;
    XClassHint		class_hints;
    XGCValues		gc_values;
    unsigned long	gc_valuemask;
    long		window_event_mask;
    int                 xpm_err;
    XpmAttributes       xpm_attr;
    char		buf[1024];

    size_hints.flags = PBaseSize | PSize | PResizeInc | PMinSize;
    size_hints.width = DEFAULT_W*(THUMB_W+50);
    size_hints.min_width = 3*(THUMB_W+50);
    size_hints.height = DEFAULT_H*(THUMB_H+50)+190;
    size_hints.min_height = 190+THUMB_H+50;
    size_hints.width_inc   = THUMB_W+50;
    size_hints.height_inc  = THUMB_H+50;
    size_hints.base_width  = 0;
    size_hints.base_height = 190;
    wm_hints.initial_state = NormalState;
    wm_hints.input = True;
    wm_hints.flags = StateHint | InputHint;

    curw = 0;
    curh = 0;

    window = XCreateSimpleWindow(display, RootWindow(display, screen),
				 0, 0, size_hints.width, size_hints.height, 2,
				 WhitePixel(display,screen),
				 BlackPixel(display,screen));

    class_hints.res_name = progname;
    class_hints.res_class = "stillgrab";

    XmbSetWMProperties(display, window, "Digital Camera Image Grabber", "Digital Camera Image Grabber", 
		       argv, argc, &size_hints, &wm_hints, &class_hints);

    window_event_mask = ExposureMask | ButtonPressMask | KeyPressMask |
	StructureNotifyMask;
    XSelectInput(display, window, window_event_mask);

    gc_values.graphics_exposures = False;
    gc_values.foreground = WhitePixel(display,screen);
    gc_values.background = BlackPixel(display,screen);
    gc_valuemask = GCGraphicsExposures | GCForeground | GCBackground;
    gc = XCreateGC(display, window, gc_valuemask, &gc_values);

    xpm_attr.valuemask = XpmCloseness;
    xpm_attr.closeness = 70000;

#define OPENXPM(NAME, XPM, XPMMASK)  \
    xpm_err = XpmReadFileToPixmap(display, window, NAME, &XPM, &XPMMASK, &xpm_attr); \
    XPMerr(xpm_err); \
    if (!XPM) { \
	fprintf(stderr, "%s: cannot read gfx bitmap file '%s'.\n", progname, NAME); \
	Exit(1); \
    } 
    

    getcwd(buf,1024);
    chdir(GFXDIR);
    OPENXPM(TILEXPM, bgtile, bgtilemask);
    OPENXPM(FRAMEXPM, frame, framemask);
    OPENXPM(HIFRAMEXPM, hiframe, hiframemask);
    OPENXPM(QUITXPM, quitxpm, quitxpmmask);
    OPENXPM(RETRIEVEXPM, retxpm, retxpmmask);
    OPENXPM(DELETEXPM, deletexpm, deletexpmmask);
    OPENXPM(SLIDEXPM, slidexpm, slidexpmmask);
    OPENXPM(SLIDE2XPM, slide2xpm, slide2xpmmask);
    chdir(buf);
}
